#!/usr/bin/perl
#
#   Bluestem - University of Illinois WWW Identification Service
#
#   doc.cgi - Bluestem API Document Serving CGI
#
#   Copyright 1996-1999 - Board of Trustees of the University of Illinois
#
#   Ed Kubaitis - ejk@uiuc.edu
#   Computing & Communications Services Office
#   University of Illinois - Urbana
#

use strict;
use warnings;
use lib "/var/www/bluestem-client/lib";
use Bluestem;
$| = 1;
#require("ctime.pl");

my(
  $C_NT, $Client, %MimeType, $DirListPE, $IconDirURL, %MimeIcons,
  $DocRoot, $Path, $ScriptName, $ID, $IdleTime, $SessionTime, %C_Doc,
  $ListDir, $cgi
  );

use vars qw(
   $NewScriptName $NewPathInfo @C_CGIFile @C_CGIDir $C_CGISfxRE @C_SfxCGIDir
   );


$C_NT = $Bluestem::C_NT;
$Client = $Bluestem::R_Client;

#-----------------------------------------------------------
#  MIME types table
#-----------------------------------------------------------
%MimeType = (
   'bin',		'application/octet-stream',
   'exe',		'application/octet-stream',
   'oda',		'application/oda',
   'pdf',		'application/pdf',
   'ai',		'application/postscript',
   'eps',		'application/postscript',
   'ps',		'application/postscript',
   'rtf',		'application/rtf',
   'mif',		'application/x-mif',
   'csh',		'application/x-csh',
   'con',		'application/x-connector',
   'dvi',		'application/x-dvi',
   'hdf',		'application/x-hdf',
   'latex',		'application/x-latex',
   'nc',		'application/x-netcdf',
   'cdf',		'application/x-netcdf',
   'sh',		'application/x-sh',
   'tcl',		'application/x-tcl',
   'tex',		'application/x-tex',
   'texinfo',		'application/x-texinfo',
   'texi',		'application/x-texinfo',
   't',			'application/x-troff',
   'tr',		'application/x-troff',
   'roff',		'application/x-troff',
   'man',		'application/x-troff-man',
   'me',		'application/x-troff-me',
   'ms',		'application/x-troff-ms',
   'src',		'application/x-wais-source',
   'zip',		'application/zip',
   'gtar',		'application/x-gtar',
   'shar',		'application/x-shar',
   'tar',		'application/x-tar',
   'hqx',		'application/mac-binhex40',
   'au',		'audio/basic',
   'snd',		'audio/basic',
   'aif',		'audio/x-aiff',
   'aiff',		'audio/x-aiff',
   'aifc',		'audio/x-aiff',
   'ra',		'audio/x-pn-realaudio',
   'ram',		'audio/x-pn-realaudio',
   'rpm',		'audio/x-pn-realaudio-plugin',
   'wav',		'audio/x-wav',
   'gif',		'image/gif',
   'ief',		'image/ief',
   'jpeg',		'image/jpeg',
   'jpg',		'image/jpeg',
   'jpe',		'image/jpeg',
   'tiff',		'image/tiff',
   'tif',		'image/tiff',
   'ras',		'image/x-cmu-raster',
   'pnm',		'image/x-portable-anymap',
   'pbm',		'image/x-portable-bitmap',
   'pgm',		'image/x-portable-graymap',
   'ppm',		'image/x-portable-pixmap',
   'rgb',		'image/x-rgb',
   'xbm',		'image/x-xbitmap',
   'xpm',		'image/x-xpixmap',
   'xwd',		'image/x-xwindowdump',
   'htm',		'text/html',
   'html',		'text/html',
   'uiuchtml',		'text/html',
   'uihtml',		'text/html',
   'nfhtml',		'text/html',
   'txt',		'text/plain',
   'rtx',		'text/richtext',
   'tsv',		'text/tab-separated-values',
   'etx',		'text/x-setext',
   'mpeg',		'video/mpeg',
   'mpg',		'video/mpeg',
   'mpe',		'video/mpeg',
   'qt',		'video/quicktime',
   'mov',		'video/quicktime',
   'viv',		'video/vivo',
   'vivo',		'video/vivo',
   'avi',		'video/x-msvideo',
   'movie',		'video/x-sgi-movie',
   'wrl',		'x-world/x-vrml',
   'map',		'magnus-internal/imagemap',
   'shtml',		'magnus-internal/parsed-html',
   'cgi',		'magnus-internal/cgi',
   'netids',		'text/plain',
   );


#-----------------------------------------------------------
#   Directory listing constants
#-----------------------------------------------------------
$DirListPE = '##DIRLIST##'; # 'place holder' Path Extension to use when
			    #  directory reference
$IconDirURL = 'https://www-s4.uiuc.edu/ns-icons';
%MimeIcons = (
   'menu',		'menu.gif',
   'text',		'text.gif',
   'image',		'image.gif',
   'audio',		'sound.gif',
   'video',		'movie.gif',
   'application',	'binary.gif',
   'unknown',		'unknown.gif',
   );


#-----------------------------------------------------------
#   Main Program
#-----------------------------------------------------------
&preset_doc;   # preset configuration parameters
$DocRoot = $C_Doc{'DocRoot'} ? $C_Doc{'DocRoot'} : '';
$Path = $Bluestem::R_PathInfo;
$ScriptName = $Bluestem::R_ScriptName;

# Other initialization
$ID = '';
$ListDir = 0;
@C_CGIFile = ();
@C_CGIDir = ();
$C_CGISfxRE = '';
@C_SfxCGIDir = ();

#---Check document path in PATH_INFO-----------------------
&abort("Invalid call to doc.cgi") unless $Path;
$Path = substr($Path,1);
&abort("Invalid call to doc.cgi") if $Path =~ m!^/!;
$Path = './' unless $Path;

#---Identify client----------------------------------------
($ID, $IdleTime, $SessionTime) = &bluestem_id;
&bluestem_login() unless $ID;
&bluestem_login('', "Idle more than $C_Doc{'MaxIdle'} seconds.")
    if  $IdleTime > $C_Doc{'MaxIdle'};

#---Move to secure document root---------------------------
chdir($DocRoot) || &abort("Can't chdir to $DocRoot");

#---Handle request-for-directory case----------------------
if (-d $Path) {
   #---if client left off trailing slash------------
   if ($Path !~ m|/$|) {
      print "Location: $Bluestem::R_MyURL/\n\n";
      exit;
      }
   if    (-r "${Path}index.html") { $Path .= 'index.html';             }
   elsif (-r "${Path}home.html")  { $Path .= 'home.html';              }
   else                           { $Path .= $DirListPE; $ListDir = 1; } 
   }


#---check client authorized--------------------------------
&check_authorized($ID, $Path);

#---Process according to type------------------------------
if    ($cgi = &check_CGI($Path)) { &execfile($cgi); }
elsif ($ListDir)                 { &listdirectory($Path); }
else                             { &sendfile($Path); }


#-----------------------------------------------------------
#   abort - abort with client message
#-----------------------------------------------------------
sub abort {
   my($msg) = @_;
   my($logmsg);

   $logmsg = &bluestem_log("$ID\@$Client: $msg");
   $msg = "<font color=#d00000> <b>$msg</b></font><br>";

   my($appServerAdminEmail) = $Bluestem::C_AppServerAdminEmail;
   $appServerAdminEmail .= "\@$Bluestem::R_ServerName"
     unless $appServerAdminEmail =~ /\@/;

   print <<"-30-";
Content-type: text/html

   <html>
   <head>
   <title>Bluestem Document Server</title>
   </head>
   <body>
   <center>
   <b><font size=+1>
   Bluestem Document Server
   </font></b><p>
   $msg
   <p>
   <b>URL</b>: <a href=$Bluestem::R_MyURL>$Bluestem::R_MyURL</a><br>
   <b>Log message</b>: $logmsg<br>
   <p>
   </center>
   This failure indicates a problem on either the particular Bluestem
   application server you're trying to use ($Bluestem::R_ServerName) or the
   CITES central Bluestem ID servers.  Contact information for both of
   these is listed below.
   <p>
   The failure may be of a temporary nature, so you may want to try
   again shortly.  To try again, either click on the URL link above
   or use your browser's Reload/Refresh button.
   <p>
   If you wish to contact someone about this failure (e.g., if the
   failure persists), please contact both of the addresses below, and
   copy/cut&paste all of the above information into your note.
   <p>
   <hr>
   <address>
   $Bluestem::R_ServerName Bluestem Application Server Admin:
   <a href=mailto:$appServerAdminEmail>$appServerAdminEmail</a>
   </address>
   <address>
   <a href=http://www.uic.edu/depts/accc/cso/ target=_blank>CSO Consultants</a>:
   <a href=mailto:consult\@uic.edu>consult\@uic.edu</a>
   </address>
   </body>
   </html>
-30-
   
   exit(1);
   }


#-----------------------------------------------------------
#   check_authorized - check if ID authorized to access file
#-----------------------------------------------------------
sub check_authorized {
   my($id, $path) = @_;
   my(@subdirs) = split(m|/|, $path);
   my(@try, $found , $idfile);
   
   while(scalar(@subdirs)) {
      $idfile = join('/', (@try, $C_Doc{'IdFile'}));
      if (-r $idfile) {
	 open(ID, $idfile) || &abort("$idfile: $!\n");
	 while(<ID>) {
	    if (m!^$id([^-a-zA-Z0-9\@/.]|$)!) {
	       $found++;
	       close(ID);
	       last;
	       }
	    }
	 &abort("Sorry, $id not authorized for $path") unless $found;
	 $found = '';
	 }
      push(@try,shift(@subdirs));
      }
   }



#-----------------------------------------------------------
#   check_CGI - check if file is a CGI
#-----------------------------------------------------------
sub check_CGI {
   my($path) = @_;
   my(@subdirs) = split(m|/|, $path);
   my($f,$d);
   my($cgi) = shift(@subdirs);

   return 0 if $C_Doc{'CGIDisabled'};
      
   while(-d $cgi) { 
      last unless scalar(@subdirs);
      $cgi = "$cgi/" . shift(@subdirs);
      }

   $NewScriptName = "/$cgi";
   $NewPathInfo = join('/', @subdirs);
   $NewPathInfo = "/$NewPathInfo" if $NewPathInfo;
      
   for $f(@C_CGIFile) { 
      return $cgi if $NewScriptName eq $f;
      }
   for $d(@C_CGIDir) {
       return $cgi if substr($NewScriptName,0,length($d)) eq $d;
       }
   if ($C_CGISfxRE && $cgi =~ /$C_CGISfxRE/) {
      for $d(@C_SfxCGIDir) { 
	 return $cgi if substr($NewScriptName,0,length($d)) eq $d;
	 }
      }
   return 0;
   }



#-----------------------------------------------------------
#   execfile - execute CGI program
#-----------------------------------------------------------
sub execfile {
   my($cgi) = @_;

   if ($C_NT) {
      $cgi = "perl.exe $cgi" if $cgi =~ /\.pl$/;
      }
   else {
      &abort("$cgi: no such program") unless -x $cgi;
      }
   
   $ENV{'SCRIPT_NAME'} = $NewScriptName;
   $ENV{'PATH_INFO'} = $NewPathInfo;
   $ENV{'REMOTE_USER'} = $ID;

   # On NT, use of exec() results in new values of SCRIPT_NAME
   # and PATH_INFO not being propagated to the CGI
   if ($C_NT) { system($cgi); exit; }
   else       { exec($cgi); }
   }


#-----------------------------------------------------------
#   listdirectory - transmit listing of directory to client
#-----------------------------------------------------------
sub listdirectory {
   my($path) = @_;
   my($mimetype, @allfiles, $file, $efile, $fullname, $icon);

   &abort("incorrect path $path") unless ($path =~ s/${DirListPE}$//);

   opendir(USERDIR, $path) || &abort("Can't open directory $path: $!");
   @allfiles = sort (grep(!/^\.\.?$/, readdir(USERDIR)));
   closedir(USERDIR);
    
   print <<"-30-";
Content-type: text/html

<HTML><HEAD><TITLE>Directory List of $path</TITLE>
</HEAD>
<BODY>
<H3>Directory List of $path</H3>
-30-

   foreach $file (@allfiles) {
      next if ($file eq $C_Doc{'IdFile'});
      $efile = $file; # URL encode the file name
      $efile =~ s/([^-0-9a-zA-Z\$_.+*'()])/"%" . uc(sprintf("%2.2x",ord($1)))/eg;
      $fullname = $path . '/' . $file;
      $icon = "$IconDirURL/";
      if (-d $fullname) {
	 $icon .= $MimeIcons{'menu'};
         print "<img src=$icon> <a href=$efile/>$file</a><BR>\n";
	 }
      else {
	 $mimetype = &mimetype($fullname);
	 $mimetype =~ s/\/.*$//;
	 $icon .= ((defined $MimeIcons{$mimetype}) 
	           ? $MimeIcons{$mimetype}
		   : $MimeIcons{'unknown'}
		   );
         print "<img src=$icon> <a href=$efile>$file</a><BR>\n";
         }
      }
   print "</BODY></HTML>\n";
   }


#-----------------------------------------------------------
#   sendfile - transmit file to client
#-----------------------------------------------------------
sub sendfile {
   my($path) = @_;
   my($mimetype) = &mimetype($path);
   my($bufsiz) = 32768;
   my($size, $buf);
   
   $path =~ m!$C_Doc{'IdFile'}$! && &abort("$path: not available");
   open(File, $path) || &abort("$path: $!");
   if ($C_NT && $mimetype !~ m|^text/|) {
      binmode(File); binmode(STDOUT);
      }
   $size = -s File;
   $size || &abort("$path: empty file");
   
   print("Content-length: $size\n");
   print("Content-type: $mimetype\n\n");
   
   while(read(File,$buf,$bufsiz)) { syswrite(STDOUT, $buf, length($buf)); }
   }



#-----------------------------------------------------------
#   mimetype - return mimetype for file suffix
#-----------------------------------------------------------
sub mimetype {
   my($path) = @_;
   my($type, $Ext);
   
   $Ext = $1 if $path =~ /\.([^.]+)$/;
   return $C_Doc{'MimeType'} unless $Ext;
   $Ext =~ tr/A-Z/a-z/;  
   return $type if ($type = $MimeType{$Ext});
   $C_Doc{'MimeType'};
   }


#-----------------------------------------------------------
#   Preset Document Server Configuration Symbols
#-----------------------------------------------------------
sub preset_doc {
   my($arrayRE) = '^(CGIFile|CGIDir|SfxCGIDir)$';

   if (!open(Conf, $Bluestem::C_DocConf)) {
      &bluestem_log("$Bluestem::C_DocConf: $!");
      exit 1;
      }

   while(<Conf>) {
      my($symbol, @v, $value, $push);
      chop;
      next if /^\s*#/;
      next if /^\s/;
      next if /^\s*$/;
      s/#.*$//;
      $_ = $1 if /^(.*)$/; # untaint doc.conf values
      ($symbol,@v) = split;
      $value = join(' ', @v);
      if ($symbol =~ /$arrayRE/) {
	 $push =  "push(\@C_$symbol, '$value');";
	 eval($push);
	 }
      else { $C_Doc{$symbol} = $value; }
      }

   $C_CGISfxRE = '\.(' . join('|', split(/,/, $C_Doc{'CGISfx'})) . ')$'
      if $C_Doc{'CGISfx'};
   }
