#!/usr/bin/perl
#
#   Bluestem - University of Illinois WWW Identification Service
#
#   lb_test.cgi - "Little Bluestem" API Reference Model Test 
#
#   Copyright 2010 - Board of Trustees of the University of Illinois
#
#   Johnathan Kupferer <jtk@uic.edu>
#   ACCC
#   University of Illinois at Chicago
#

use strict;
use warnings;
use lib "/var/www/bluestem-client/lib";
use LBluestem;

my(
  $C_IdleMax, $Client, $ClientIP, $ClientCookie, $CacheID, $clientrandom,
  $script_name, $ThisHost, $ClientCookieName, $LogoutURL, $ID, $IdleTime,
  $SessionTime, $FullID, $ShortID, $User, $Domain, $Auth, $todc, $pid,
  $LoginTime, $ApplServerCache, $k, $v, @EnvVars
  );

#-----------------------------------------------------------
#   Configuration
#-----------------------------------------------------------
$C_IdleMax = 300;
#-----------------------------------------------------------


#-----------------------------------------------------------
#   Main Program
#-----------------------------------------------------------

require("ctime.pl");

#---CGI Info------------------------------------------------
$Client = $ENV{'REMOTE_HOST'};
$ClientIP = $ENV{'REMOTE_ADDR'};
$Client = $ClientIP unless $Client;
$ClientCookie = &LBluestem::cookie_read();
#   suppress display of ClientRandom secret
$ClientCookie =~ s/~[0-9a-f]+$/~****************/;
($CacheID,$clientrandom) = split(/~/, $ClientCookie);
$ClientCookieName = "Bluestem_$LBluestem::C_ServerNamePortCookie";

$script_name = $LBluestem::C_ScriptName;
$script_name =~ s|/[^/]+$|/logout|;
$script_name .= (($LBluestem::C_NT) ? '.pl' : '.cgi');
$ThisHost = $LBluestem::C_ServerNamePort;
$LogoutURL = "https://$ThisHost$script_name";


#---Valid API Calls-----------------------------------------
($ID, $IdleTime, $SessionTime) = &lb_id;
&lb_login unless $ID; 
&lb_relogin if $IdleTime > $C_IdleMax;

$FullID = &lb_id_full($ID);
$ShortID = &lb_id_short($FullID);
($User,$Domain,$Auth) = &lb_id_parse($ID);

$IdleTime = &hhmmss($IdleTime);
$SessionTime = &hhmmss($SessionTime);
($todc, $pid) = split(/-/, $CacheID);
$LoginTime = &ctime($todc);

#---Peeking under the hood----------------------------------
open(Cache, "$LBluestem::C_CacheDir/$CacheID");
$ApplServerCache = <Cache>;
close(Cache);
#   suppress display of ClientRandom secret
$ApplServerCache =~ s/,[0-9a-f]{16}\b/,****************/;

#---Environment Variables-----------------------------------
for $k(sort(keys(%ENV))) {
   $v = $ENV{$k};
   $v =~ s/(~|%7E)[0-9a-f]{16}/$1****************/g if $k eq 'HTTP_COOKIE';
   push(@EnvVars,"<tr><td><code>$k</code></td><td><code>$v</code></td></tr>\n");
   }

print "Content-type: text/html\n\n";
print <<"-30-";
   <html>
   <head>
   <title>Little Bluestem API Test</title>
   </head>
   <body>
   <center>
   <a href=http://www.uic.edu/>
   <img src="$LBluestem::C_ImgPath/UIbar.gif" border="none" width=468 height=20></a><br>
   <b><font size=+1>Little Bluestem API Test</font></b><br>
   <img src="$LBluestem::C_ImgPath/UIthinbar.gif" width=468 height=6><br>
   <a href=$LogoutURL><b>Logout from $ThisHost</b></a>
   <p>
   <table border>
   <tr><th>Bluestem Item</th>        <th>Value</th></tr>
   <tr><td>ID</td>                   <td align=center><b>$ID</b></td></tr>
   <tr><td>ID (short)</td>           <td align=center><b>$ShortID</b></td></tr>
   <tr><td>ID (full)</td>            <td align=center><b>$FullID</b></td></tr>
   <tr><td>User</td>                 <td align=center><b>$User</b></td></tr>
   <tr><td>Domain</td>               <td align=center><b>$Domain</b></td></tr>
   <tr><td>Authentication Method</td>
       <td align=center><b>$Auth</b></td></tr>
   <tr><td>Client</td>               
       <td align=center><b>$Client($ClientIP)</b></td></tr>
   <tr><td>Cache-ID</td>             
       <td align=center><b>$CacheID</b></td></tr>
   <tr><td>Client Cookie Name</td>  
       <td align=center><b><font size=-1>$ClientCookieName</font></b></td></tr>
   <tr><td>Client Cookie Value</td>        
       <td align=center><b><font size=-1>$ClientCookie</font></b></td></tr>
   <tr><td>Application Server Cache</td>   
       <td align=center><b><font size=-1>$ApplServerCache</font></b></td></tr>
   <tr><td>Login Time</td>           
       <td align=center><b>$LoginTime</b></td></tr>
   <tr><td>Session Time (hh:mm:ss)</td>
       <td align=center><b>$SessionTime</b></td></tr>
   <tr><td>Idle Time (hh:mm:ss)</td>
       <td align=center><b>$IdleTime</b></td></tr>
   </table>
   <p>
   <table border>
   <tr><th>Environment Variable</th><th>Value</th></tr>
   @EnvVars
   </table>
   </center>
   <hr>
   <address>UIC WWW Identification Service -
   <a href=mailto:www\@uic.edu>www\@uic.edu</a>
   </address>
   </body>
   </html>
-30-

exit;

sub hhmmss {
   my($secs) = @_;
   my($hr, $mi, $sc, $hhmmss);
   
   $sc = int($secs%60);
   $mi = int(int($secs/60)%60);
   $hr = int($secs/3600);
   
   $hhmmss = sprintf("%d:%02d:%02d", $hr, $mi, $sc);
   $hhmmss =~ s/^[0:]+//;
   $hhmmss = '0' unless $hhmmss;
   $hhmmss;
   }

